@@ -1,6 +1,6 @@
 package Munin::Common::Config;
 
-# $Id: Config.pm 3872 2010-08-20 12:12:55Z janl $
+# $Id$
 
 use warnings;
 use strict;
@@ -38,7 +38,15 @@ my %legal = map { $_ => 1 } (
 	"onlynullcdef", "group_order", "pipe", "pipe_command",
 	"unknown_limit", "num_unknowns", "dropdownlimit",
 	"max_graph_jobs", "max_cgi_graph_jobs", "munin_cgi_graph_jobs",
-        "max_html_jobs", );
+	"max_html_jobs", "cgitmpdir", "update_rate",
+	"max_size_x", "max_size_y", 
+	"staticdir", "html_strategy",
+	"rrdcached_socket", "graph_args_after",
+    "graph_future", "trend", "predict",
+	"html_rename",
+	"worker_start_delay",
+	"num_messages",
+	);
 
 my %bools = map { $_ => 1} qw(yes no true false on off 1 0);
 
@@ -90,10 +98,13 @@ sub _trim {
 }
 
 
+# allows # characters to get through as long as they're escaped
+# with a backslash
 sub _strip_comment {
     my $class = shift;
 
-    $_[0] =~ s/#.*//;
+    $_[0] =~ s/(?<!\\)#.*//;
+    $_[0] =~ s/\\#/#/g;
 
     return;
 }
