@@ -1,6 +1,6 @@
 package Munin::Common::TLS;
 
-# $Id: TLS.pm 3999 2010-11-12 19:35:38Z bldewolf $
+# $Id$
 
 use warnings;
 use strict;
@@ -373,7 +373,7 @@ sub read {
     my $err = &Net::SSLeay::print_errs("");
     if (defined $err and length $err) {
         $self->{logger}("[TLS] Warning in read: $err");
-        return undef;
+        return;
     }
     undef $read if($read eq ''); # returning '' signals EOF
 
