@@ -161,7 +161,7 @@ my $only_fqn = '';
 
 my $watermark = "Munin " . $Munin::Common::Defaults::MUNIN_VERSION;
 
-# RRD param for --daemon
+# RRD param for RRDCACHED_ADDRESS
 my @rrdcached_params;
 
 my $running     = 0;
@@ -308,8 +308,9 @@ sub graph_startup {
 
     if ($config->{"rrdcached_socket"}) { 
 	    if ($RRDs::VERSION >= 1.3){
-		    push @rrdcached_params, "--daemon";
-		    push @rrdcached_params, $config->{"rrdcached_socket"}; 
+		# Using the RRDCACHED_ADDRESS environnement variable, as
+                # it is way less intrusive than the command line args.
+                $ENV{RRDCACHED_ADDRESS} = $config->{"rrdcached_socket"};
 	    } else { 
 		    ERROR "[ERROR] RRDCached feature ignored: RRD version must be at least 1.3. Version found: " . $RRDs::VERSION; 
 	    }
@@ -333,7 +334,7 @@ sub graph_startup {
 sub graph_check_cron {
 
     # Are we running from cron and do we have matching graph_strategy
-    if (&munin_get($config, "graph_strategy", "cgi") ne "cron" and $cron) {
+    if (&munin_get($config, "graph_strategy", "cron") ne "cron" and $cron) {
 
         # Strategy mismatch: We're run from cron, but munin.conf says
         # we use dynamic graph generation
@@ -1422,6 +1423,9 @@ sub process_service {
             # Otherwise, it will be difficult to debug post-mortem
             ERROR "[RRD ERROR] rrdtool 'graph' '" . join("' \\\n\t'", @rrdcached_params, @complete) . "'\n";
         }
+        elsif (!-f $picfilename) {
+		ERROR "[RRD ERROR] rrdtool graph did not generate the image (make sure there are data to graph).\n";
+        }
         else {
 
             # Set time of png file to the time of the last update of
