@@ -63,7 +63,7 @@ use Exporter;
 
 our (@ISA, @EXPORT);
 @ISA    = qw(Exporter);
-@EXPORT = qw(html_startup html_main get_config emit_main_index emit_comparison_template emit_group_template emit_graph_template emit_service_template emit_category_template emit_problem_template);
+@EXPORT = qw(html_startup html_main get_config emit_main_index emit_comparison_template emit_group_template emit_graph_template emit_service_template emit_category_template emit_problem_template update_timestamp);
 
 use HTML::Template;
 use POSIX qw(strftime);
@@ -100,6 +100,18 @@ my $running=0;
 my $timestamp;
 my $htmlconfig;
 
+sub update_timestamp {
+    # For timestamping graphs
+    $timestamp = strftime("%Y-%m-%d %T%z (%Z)", localtime);
+    if ($timestamp =~ /%z/) {
+	# %z (numeric timezone offset) may not be available, but %Z
+	# (timeszone name) seems to be universaly supported though the
+	# timezone names are not really standardized.
+	$timestamp = strftime("%Y-%m-%d %T%Z", localtime);
+    }
+    $htmltagline = "This page was generated by <a href='http://munin-monitoring.org/'>Munin</a> version ".$Munin::Common::Defaults::MUNIN_VERSION." at $timestamp";
+}
+
 sub html_startup {
     my ($args) = @_;
     local @ARGV = @{$args};
@@ -133,15 +145,7 @@ sub html_startup {
 
     $max_running = &munin_get($config, "max_html_jobs", $max_running);
 
-    # For timestamping graphs
-    $timestamp = strftime("%Y-%m-%d %T%z (%Z)", localtime);
-    if ($timestamp =~ /%z/) {
-	# %z (numeric timezone offset) may not be available, but %Z
-	# (timeszone name) seems to be universaly supported though the
-	# timezone names are not really standardized.
-	$timestamp = strftime("%Y-%m-%d %T%Z", localtime);
-    }
-    $htmltagline = "This page was generated by <a href='http://munin-monitoring.org/'>Munin</a> version ".$Munin::Common::Defaults::MUNIN_VERSION." at $timestamp";
+    update_timestamp();
  
     %comparisontemplates = instanciate_comparison_templates($tmpldir);
 }
@@ -269,6 +273,24 @@ sub emit_comparison_template {
 		push((@$comparepeers), {"name" => $peer->{"name"}, "link" => $comparelink});
 	}
 
+    # when comparing categories, we're generating the page inside the
+    # domain, but the images' urls will point inside the node itself,
+    # or even worse within a category inside it. We strip out the
+    # extra '../' that is used to generate a relative path which is no
+    # longer valid.
+    foreach my $cat(@{$key->{'comparecategories'}}) {
+        foreach my $service(@{$cat->{'services'}}) {
+            foreach my $node(@{$service->{'nodes'}}) {
+                foreach my $imgsrc(qw(imgday imgweek imgmonth imgyear
+                              cimgday cimgweek cimgmonth cimgyear
+                              zoomday zoomweek zoommonth zoomyear)) {
+                    next unless defined($node->{$imgsrc});
+                    $node->{$imgsrc} =~ s|^\.\./\.\./(?:\.\./)?|../|;
+                }
+            }
+        }
+    }
+
     $comparisontemplates{$t}->param(
                                     INFO_OPTION => 'Groups on this level',
                                     NAME        => $key->{'name'},
@@ -373,6 +395,16 @@ sub emit_category_template {
 
     DEBUG "[DEBUG] Creating global category page ".$filename;
 
+    foreach my $graphs(@{$key->{'graphs'}}) {
+        foreach my $graph(@{$graphs->{'graphs'}}) {
+            foreach my $imgsrc(qw(imgday imgweek imgmonth imgyear
+                              cimgday cimgweek cimgmonth cimgyear
+                              zoomday zoomweek zoommonth zoomyear)) {
+                $graph->{$imgsrc} =~ s|^(?:\.\./)+||
+            }
+        }
+    }
+
     $graphtemplate->param(
                           PATH        => $key->{'path'},
                           CSS_NAME    => get_css_name(),
