@@ -26,7 +26,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Id: LimitsOld.pm 4130 2011-01-20 15:51:37Z jo $
+$Id$
 
 =end comment
 
@@ -44,8 +44,7 @@ our (@ISA, @EXPORT);
 use POSIX qw ( strftime );
 use Getopt::Long;
 use Time::HiRes;
-use Text::Balanced qw ( extract_multiple extract_delimited
-    extract_quotelike extract_bracketed );
+use Text::Balanced qw ( extract_bracketed );
 use Log::Log4perl qw ( :easy );
 
 use Munin::Master::Logger;
@@ -60,6 +59,7 @@ my @limit_hosts    = ();
 my @limit_services = ();
 my @limit_contacts = ();
 my $stdout         = 0;
+my $force_run_as_root = 0;
 my %notes          = ();
 my $force          = 0;
 my $config;
@@ -88,6 +88,7 @@ sub limits_startup {
         "debug!"    => \$DEBUG,
         "stdout!"   => \$stdout,
         "force!"    => \$force,
+        "force-run-as-root!" => \$force_run_as_root,
         "version!"  => \$do_version,
         "help"      => \$do_usage
         );
@@ -95,7 +96,7 @@ sub limits_startup {
     print_usage_and_exit()   if $do_usage;
     print_version_and_exit() if $do_version;
 
-    exit_if_run_by_super_user();
+    exit_if_run_by_super_user() unless $force_run_as_root;
 
     $config = &munin_config($conffile);
 
@@ -105,6 +106,9 @@ sub limits_startup {
 
 
 sub limits_main {
+    # We're liable to receive SIGPIPEs if the given commands don't work
+    $SIG{PIPE} = 'IGNORE';
+
     my $update_time = Time::HiRes::time;
 
     my $lockfile = "$config->{rundir}/munin-limits.lock";
@@ -121,6 +125,8 @@ sub limits_main {
 
     process_limits();
 
+    close_pipes();
+
     &munin_writeconfig("$config->{dbdir}/limits", \%notes);
 
     $update_time = sprintf("%.2f", (Time::HiRes::time - $update_time));
@@ -130,6 +136,16 @@ sub limits_main {
     INFO "[INFO] munin-limits finished ($update_time sec)";
 }
 
+sub close_pipes {
+    foreach my $cont (@{munin_get_children($config->{"contact"})}) {
+        if($cont->{pipe}) {
+            my $c = munin_get_node_name($cont);
+
+            DEBUG "[DEBUG] Closing pipe for contact $c";
+            close $cont->{pipe} or WARN "[WARNING] Failed to close pipe for contact $c: $!";
+        }
+    }
+}
 
 sub process_limits {
 
@@ -250,8 +266,8 @@ sub get_full_group_path {
     my $parent     = munin_get_parent($group);
     my $name       = get_notify_name($group);
 
-    if (defined $parent and munin_get_node_name($parent) ne "#%#root") {
-	return (get_full_group_path($parent) . " :: " . $name);
+    if (defined $parent and munin_get_node_name($parent) ne "root") {
+	return (get_full_group_path($parent) . "-" . $name);
     } else {
 	return $name;
     }
@@ -260,6 +276,7 @@ sub get_full_group_path {
 sub process_service {
     my $hash       = shift || return;
     my $hobj       = get_host_node($hash);
+    my $host       = get_notify_name($hobj);
     my $service    = munin_get_node_name($hash);
     my $hparentobj = munin_get_parent($hobj);
     my $parent     = munin_get_node_name($hobj);
@@ -269,7 +286,7 @@ sub process_service {
     if (!ref $hash) {
 	LOGCROAK("I was passed a non-hash!");
     }
-
+    return if (@limit_hosts and !grep (/^$host$/, @limit_hosts));
     return if (@limit_services and !grep (/^$service$/, @limit_services));
 
     DEBUG "[DEBUG] processing service: $service";
@@ -278,11 +295,15 @@ sub process_service {
     $hash->{'fields'} = join(' ', map {munin_get_node_name($_)} @$children);
     $hash->{'plugin'} = $service;
     $hash->{'graph_title'} = get_full_service_name($hash);
-    $hash->{'host'}  = get_notify_name($hobj);
+    $hash->{'host'}  = $host;
     $hash->{'group'} = get_full_group_path($hparentobj);
     $hash->{'worst'} = "ok";
     $hash->{'worstid'} = 0 unless defined $hash->{'worstid'};
 
+    my $state_file = sprintf ('%s/state-%s-%s.storable', $config->{dbdir}, $hash->{group}, $hash->{host}); 
+    DEBUG "[DEBUG] state_file: $state_file";
+    my $state = munin_read_storable($state_file) || {};
+
     foreach my $field (@$children) {
         next if (!defined $field or ref($field) ne "HASH");
         my $fname   = munin_get_node_name($field);
@@ -299,14 +320,35 @@ sub process_service {
         next if (!defined $warn and !defined $crit);
 
         DEBUG "[DEBUG] processing field: " . join('::', @$fpath);
+        DEBUG "[DEBUG] field: " . munin_dumpconfig_as_str($field);
+	my $value;
+    	{
+		my $rrd_filename = munin_get_rrd_filename($field);
+		my ($current_updated_timestamp, $current_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{current} || [ ] };
+		my ($previous_updated_timestamp, $previous_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{previous} || [ ] };
 
-        my $filename = munin_get_rrd_filename($field);
-        my $value    = munin_fetch("$filename");
-
-	DEBUG "[DEBUG] rrd filename is: $filename";
+		my $heartbeat = 600; # XXX - $heartbeat is a fixed 10 min (2 runs of 5 min).
+		if (! $field->{type} || $field->{type} eq "GAUGE" || $field->{type} eq "ABSOLUTE") {
+			$value = $current_updated_value;
+		} elsif (! defined $current_updated_value || ! defined $previous_updated_value || $current_updated_timestamp == $previous_updated_timestamp) {
+			# No derive computing possible. Report unknown.
+			$value = "U";
+		} elsif (time > $current_updated_timestamp + $heartbeat) {
+			# Current value is too old. Report unknown. 
+			$value = "U";
+		} elsif ($current_updated_timestamp > $previous_updated_timestamp + $heartbeat) {
+			# Old value is too old. Report unknown. 
+			$value = "U";
+		} elsif ($field->{type} eq "COUNTER" && $current_updated_value < $previous_updated_value) {
+			# COUNTER never decrease. Report unknown.
+			$value = "U";
+		} else {
+			$value = ($current_updated_value - $previous_updated_value) / ($current_updated_timestamp - $previous_updated_timestamp);
+		}
+	}
 
         # De-taint.
-        if (!defined $value) {
+        if (!defined $value || $value eq "U") {
             $value = "unknown";
         }
         else {
@@ -576,8 +618,10 @@ sub generate_service_message {
     foreach my $c (split(/\s+/, $contactlist)) {
         next if $c eq "none";
         my $contactobj = munin_get_node($config, ["contact", $c]);
-        next unless defined $contactobj;
-        next unless defined munin_get($contactobj, "command", undef);
+        if(!defined $contactobj) {
+            WARN("[WARNING] Missing configuration options for contact $c; skipping");
+            next;
+        }
         if (@limit_contacts and !grep (/^$c$/, @limit_contacts)) {
             next;
         }
@@ -589,8 +633,12 @@ sub generate_service_message {
         if (!$hash->{'state_changed'} and !$obsess) {
             next;    # No need to send notification
         }
-        DEBUG "state has changed, notifying $c";
-        my $precmd = munin_get($contactobj, "command");
+        DEBUG "[DEBUG] state has changed, notifying $c";
+        my $precmd = munin_get($contactobj, "command", undef);
+        if(!defined $precmd) {
+            WARN("[WARNING] Missing command option for contact $c; skipping");
+            next;
+        }
         my $pretxt = munin_get(
             $contactobj,
             "text",
@@ -603,113 +651,59 @@ sub generate_service_message {
         $txt =~ s/\\n/\n/g;
         $txt =~ s/\\t/\t/g;
 
-        # In some cases we want to reopen the command
+        if($cmd =~ /^\s*([|><]+)/) {
+            WARN "[WARNING] Found \"$1\" at beginning of command.  This should no longer be necessary and will be removed from the command before execution";
+            $cmd =~ s/^\s*[|><]+//;
+        }
+
         my $maxmess = munin_get($contactobj, "max_messages", 0);
         my $curmess = munin_get($contactobj, "num_messages", 0);
         my $curcmd  = munin_get($contactobj, "pipe_command", undef);
         my $pipe    = munin_get($contactobj, "pipe",         undef);
         if ($maxmess and $curmess >= $maxmess) {
-            close($pipe);
+            DEBUG "[DEBUG] Resetting pipe for $c because max messages was reached";
+            close($pipe) or WARN "[WARNING] Failed to close pipe for $c: $!";
             $pipe = undef;
-            munin_set_var_loc($contactobj, ["pipe"], undef);
-            DEBUG "[DEBUG] Closing \"$c\" -> command (max number of messages reached).";
+            munin_set($contactobj, "pipe", undef);
         }
         elsif ($curcmd and $curcmd ne $cmd) {
-            close($pipe);
+            DEBUG "[DEBUG] Resetting pipe for $c because the command has changed";
+            close($pipe) or WARN "[WARNING] Failed to close pipe for $c: $!";
             $pipe = undef;
-            munin_set_var_loc($contactobj, ["pipe"], undef);
-            DEBUG "[DEBUG] Closing \"$c\" -> command (command has changed).";
+            munin_set($contactobj, "pipe", undef);
         }
 
         if (!defined $pipe) {
-            my @cmd = extract_multiple(
-                message_expand($hash, $cmd),
-                [sub {extract_delimited($_[0], q{"'})}, qr/\S+/],
-                undef, 1
-            );
-            @cmd = map {
-                my $c = $_;
-                $c =~ s/['"]$//;
-                $c =~ s/^['"]//;
-                $c;
-            } @cmd;
-            $contactobj->{"num_messages"} = 0;
-            if ($cmd[0] eq "|") {
-                $cmd[0] = "|-";
-            }
-            elsif ($cmd[0] !~ /^[|>]/) {
-                unshift(@cmd, "|-");
-            }
-            DEBUG("[DEBUG] opening \"$c\" for writing: \""
-		  . join('" "', @cmd)
-		  . "\".");
-            if ($cmd[0] eq ">") {
-                ## no critic
-                if (!open($pipe, join(' ', @cmd))) {
-                    FATAL("[FATAL] Could not open "
-			  . join(' ', @cmd[1 .. $#cmd])
-			  . " for writing: $!");
-                    exit 3;
-                }
-                ## critic
-            }
-            else {
-                my $pid = open($pipe, "|-");
-                if (!defined $pid) {
-                    FATAL "Fatal: Unable to  fork: $!";
-                    exit 3;
-                }
-                if (!$pid) {    # Child
-                                # Fork of stdout-to-log filter
-                    my $logstdout;
-                    my $logstderr;
-                    my $logpid = open($logstdout, "|-");
-                    if (!defined $logpid) {
-                        FATAL "Fatal: Unable to  fork: $!";
-                        exit 3;
-                    }
-                    if (!$logpid) {    # Child
-                        while (<STDIN>) {
-                            chomp;
-                            INFO "Command \"$c\" stdout: $_";
-                        }
-                        exit 0;
-                    }
-                    close(STDOUT);
-                    *STDOUT = \$logstdout;
-                    $logpid = open($logstderr, "|-");
-                    if (!defined $logpid) {
-                        FATAL "Fatal: Unable to  fork: $!";
-                        exit 3;
-                    }
-                    if (!$logpid) {    # Child
-                        while (<STDIN>) {
-                            chomp;
-                            FATAL "Command \"$c\" stderr: $_";
-                        }
-                        exit 0;
-                    }
-                    open(STDOUT, ">&", $logstdout);
-                    open(STDERR, ">&", $logstderr);
+            DEBUG "[DEBUG] Opening pipe for $c";
+            pipe(my $r, my $w) or WARN "[WARNING] Failed to open pipe for $c: $!";
+            my $pid = fork();
+            defined($pid) or WARN "[WARNING] Failed fork for pipe for $c: $!";
+            if($pid) { # parent
+                DEBUG "[DEBUG] Opened pipe for $c as pid $pid";
 
-                    exec(@cmd[1 .. $#cmd])
-                        or WARN("[WARNING] Could not run command \""
-                            . join(' ', @cmd[1 .. $#cmd])
-                            . "\": $!");
-                    exit 5;
-
-                    # NOTREACHED
-                }
+                close $r;
+                $pipe = $w;
+                munin_set($contactobj, "pipe_command", $cmd);
+                munin_set($contactobj, "pipe",         $pipe);
+                munin_set($contactobj, "num_messages", 0);
+                $curmess = 0;
+            } else { # child
+                close $w;
+                open(STDIN, "<&", $r);
+                exec($cmd) or WARN "[WARNING] Failed to exec for contact $c in pid $$";
+                exit;
             }
-            munin_set_var_loc($contactobj, ["pipe_command"], $cmd);
-            munin_set_var_loc($contactobj, ["pipe"],         $pipe);
         }
-        DEBUG "[DEBUG] sending message: \"$txt\"";
-        print $pipe $txt, "\n" if (defined $pipe);
-        DEBUG "[DEBUG] explicitely closing pipe as suggested by schamane on #732";
-        close $pipe if (defined $pipe);
-        $contactobj->{"num_messages"}
-            = 1 + munin_get($contactobj, "num_messages", 0);   # $num_messages++
+
+        DEBUG "[DEBUG] sending message to $c: \"$txt\"";
+        if(defined $pipe and !print $pipe $txt, "\n") {
+            WARN "[WARNING] Writing to pipe for $c failed: $!";
+            close($pipe) or WARN "[WARNING] Failed to close pipe for $c: $!";
+            $pipe = undef;
+            munin_set($contactobj, "pipe", undef);
+        }
+
+        munin_set($contactobj, "num_messages", $curmess + 1);
     }
 }
 
