@@ -974,11 +974,30 @@ sub munin_dumpconfig {
     $Data::Dumper::Indent = $indent;
 }
 
+sub purge_cycles($)
+{
+	my @topurge = (shift);
+	while (my $item = shift @topurge) {
+		if (ref($item) eq "HASH") {
+			push @topurge, values %$item;
+			%$item = ();
+		} elsif (ref($item) eq 'ARRAY') {
+			push @topurge, @$item;
+			@$item = ();
+		}
+	}
+}
+
 sub munin_refreshconfig {
     my $config   = shift;
     my @stat = stat("$config->{dbdir}/datafile");
     if ($config->{'#%#datafile_mtime'} && $stat[9] > $config->{'#%#datafile_mtime'}) {
-	$config = munin_readconfig("$config->{dbdir}/datafile");
+	# keep dbdir, as we don't reload that part
+	my $dbdir = $config->{dbdir};
+	# purge cyclic current config
+	purge_cycles($config);
+	$config = munin_readconfig("$dbdir/datafile");
+	$config->{dbdir} = $dbdir;
     }
 
     $config->{'#%#datafile_mtime'} = $stat[9];
